package com.blackberry.workspaces.api.sdk.examples;

import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.HashSet;
import java.util.Set;

import com.blackberry.workspaces.api.sdk.APISession;
import com.blackberry.workspaces.api.sdk.enums.DeviceType;
import com.blackberry.workspaces.api.sdk.json.ActivityLogRecordJson;
import com.blackberry.workspaces.api.sdk.json.BaseJson;
import com.blackberry.workspaces.api.sdk.json.BulkOperationResultJson;
import com.blackberry.workspaces.api.sdk.json.DeleteDocumentsSelectionSdsJson;
import com.blackberry.workspaces.api.sdk.json.DeleteDocumentsSelectionVdrJson;
import com.blackberry.workspaces.api.sdk.json.DocumentJson;
import com.blackberry.workspaces.api.sdk.json.GetDocumentActivityLogRequestJson;
import com.blackberry.workspaces.api.sdk.json.ItemListJson;
import com.blackberry.workspaces.api.sdk.json.ListDocumentsVdrJson;
import com.blackberry.workspaces.api.sdk.json.PagingItemListJson;
import com.blackberry.workspaces.api.sdk.json.SubmitDocumentResultJson;
import com.blackberry.workspaces.api.sdk.json.SubmitDocumentSdsJson;
import com.blackberry.workspaces.api.sdk.json.SubmitDocumentsVdrJson;
import com.blackberry.workspaces.api.sdk.json.UpdateDocumentJson;
import com.blackberry.workspaces.api.sdk.resource.Files;
import com.blackberry.workspaces.api.sdk.resource.Workspaces;

/**
 * Examples of Workspaces Files functionality
 * 
 * @author tasfaw
 *
 */
public class FilesExample
{
    private APISession apiSession;

    /**
     * Creates an instances of the FilesExample class
     * 
     * @param apiSession An ApiSession instance to be used.
     */
    public FilesExample(APISession apiSession)
    {
        this.apiSession = apiSession;
    }

  /**
   * Creates a new document and uploads it to the specified room.
   * 
   * @param authToken
   *          an auth token(beginning with 'expires=")
   * @param roomId
   *          denotes the id for the workspace on the WatchDox server
   * @param filePath
   *          fully qualified path to the file to be uploaded
   * @return - the guid for the newly uploaded document
   * 
   * @throws IOException
   * @throws ParseException
   */
  public String createDocument(String roomId, String filePath)
      throws IOException, ParseException
  {
    // START SNIPPET: devGuide7

    Files files = apiSession.getFilesResource();

    String docGuid = "";

    // Make a call to generate a guid for the document
    String uploadGuid = files.generateGuidV30().getGuid();

    // Set up a file object and add the content disposition that WatchDox expects
    File fileObject = new File(filePath);
    String fileName = fileObject.getName();

    try
    {
        Workspaces workspaces = apiSession.getWorkspacesResource();
      // Upload the document to WatchDox
        workspaces.uploadDocumentByPostV30(uploadGuid, roomId, fileObject,
          DeviceType.UPLOADER, fileName, fileName, null);

      // Set up an object with details for submitting the uploaded document
      Set<String> guidSet = new HashSet<String>();
      guidSet.add(uploadGuid);

      SubmitDocumentsVdrJson submitDocumentsVdrJson = new SubmitDocumentsVdrJson();
      submitDocumentsVdrJson.setDocumentGuids(guidSet);

      // Submit the document
      ItemListJson<SubmitDocumentResultJson> response =
         workspaces.submitDocumentsV30(roomId, submitDocumentsVdrJson);

      // The guid for the new document is not the same as what was used when uploading.
      // Process the response for the submitted document guid,
      // which will be used for subsequent operations.
      // Note, this example assumed only one item was returned in the response.
      SubmitDocumentResultJson docResultJson = response.getItems().get(0);
      docGuid = docResultJson.getGuid();
    }
    // END SNIPPET: devGuide7
    catch (Exception e)
    {
      System.out.println(e);
    }

    return docGuid;
  }

  /**
   * Creates document in exchange.
   * @param filePath
   *          fully qualified path to the file to be uploaded
   * @return the guid for the newly uploaded doc
   *
   * @throws IOException
   * @throws ParseException
   */
  public String createDocumentExchange(String filePath)
      throws IOException, ParseException
  {
    // START SNIPPET: devGuide8

    Files files = apiSession.getFilesResource();

    String docGuid = "";

    // Make a call to generate a guid for the document
    String uploadGuid = files.generateGuidV30().getGuid();

    // Set up a file object and add the content disposition that WatchDox expects
    File fileObject = new File(filePath);
    String fileName = fileObject.getName();

    try
    {
      // Upload the document to WatchDox
    	files.uploadDocumentByPostV30(uploadGuid, fileObject,
          DeviceType.UPLOADER, fileName, fileName, null);

      // Set up an object with details for submitting the uploaded document
      Set<String> guidSet = new HashSet<String>();
      guidSet.add(uploadGuid);

      SubmitDocumentSdsJson submitDocumentSdsJson = new SubmitDocumentSdsJson();
      submitDocumentSdsJson.setDocumentGuids(guidSet);

      // Submit the document
      ItemListJson<DocumentJson> response =
          files.submitDocumentsV30(submitDocumentSdsJson);

      // The guid for the new document is not the same as what was used when uploading.
      // Process the response for the submitted document guid,
      // which will be used for subsequent operations.
      // Note, this example assumed only one item was returned in the response.
      DocumentJson docResultJson = response.getItems().get(0);
      docGuid = docResultJson.getGuid();
    }
    // END SNIPPET: devGuide8
    catch (Exception e)
    {
      System.out.println(e);
    }

    return docGuid;
  }

  /**
   * Renames a given document
   * 
   * @param authToken
   *          authentication token
   * @param guid
   *          identifies the document to be renamed
   * @param newName
   *          the new name for the document
   * @param roomId
   *          denotes the id for the workspace on the WatchDox server
   */
  public void renameDocument(String guid, String newName, String roomId)
  {
    Workspaces workspaces = apiSession.getWorkspacesResource();
    UpdateDocumentJson json = new UpdateDocumentJson();
    json.setDeviceType(DeviceType.UPLOADER);
    json.setNewFileName(newName);

    workspaces.renameDocumentV30(guid, roomId, json);
  }

  /**
   * Prints a list of documents in the given room
   * 
   * @param authToken
   *          authentication token
   * @param roomId
   *          denotes the id for the workspace on the WatchDox server
   */
  public void listDocumentsForRoom(String roomId)
  {
    // START SNIPPET: devGuide9

    Workspaces workspaces = apiSession.getWorkspacesResource();

    // Create an object to specify the details of what documents to list and how
    // they are returned. A few options are shown here.
    ListDocumentsVdrJson selectionJson = new ListDocumentsVdrJson();
    selectionJson.setOrderAscending(Boolean.FALSE);
    selectionJson.setFolderPath("/exampleFolder");

    // Call the list method
    PagingItemListJson<BaseJson> response =
      workspaces.listDocumentsV30(roomId, selectionJson);
    // END SNIPPET: devGuide9

    // We know there's only 1 document so just get(0). If more, then iterate the List.
    System.out.println("Document list: " + response.getItems().get(0));
  }

  /**
   * Deletes the specified document from the room
   * 
   * @param authToken
   *          - the authentication token
   * @param guidsForDeletion
   *          - a Set of GUID's for the documents to be deleted
   * @param roomId
   *          - denotes the id for the workspace on the WatchDox server
   */
  public BulkOperationResultJson deleteDocument(
      HashSet<String> guidsForDeletion, String roomId)
  {
    // START SNIPPET: devGuide14

    Workspaces workspaces = apiSession.getWorkspacesResource();

    // Create an object to specify the documents to delete and if that will be permanent.
    DeleteDocumentsSelectionVdrJson documentGuidsJson =
        new DeleteDocumentsSelectionVdrJson();

    // Set permanent document deletion
    documentGuidsJson.setIsPermanent(Boolean.TRUE);

    // The set of guids for the documents to delete
    documentGuidsJson.setDocumentGuids(guidsForDeletion);

    // Call the delete method
    BulkOperationResultJson bulkOperationResultJson;
    bulkOperationResultJson =
    		workspaces.deleteDocumentsV30(roomId, documentGuidsJson);
    // END SNIPPET: devGuide14

    return bulkOperationResultJson;
  }

  /**
   * Deletes document from exchange.
   * @param authToken
   *          the authentication token
   * @param guidsForDeletion
   *          a Set of GUID's for the documents to be deleted
   */
  public BulkOperationResultJson deleteDocumentExchange(HashSet<String> guidsForDeletion)
  {
    // START SNIPPET: devGuide15

    Files files = apiSession.getFilesResource();

    // Create an object to specify the documents to delete and if that will be permanent.
    DeleteDocumentsSelectionSdsJson documentGuidsJson =
        new DeleteDocumentsSelectionSdsJson();

    // Set permanent document deletion
    documentGuidsJson.setIsPermanent(Boolean.TRUE);

    // The set of guids for the documents to delete
    documentGuidsJson.setDocumentGuids(guidsForDeletion);

    // Call the delete method
    BulkOperationResultJson bulkOperationResultJson;
    bulkOperationResultJson = files.deleteDocumentsV30(documentGuidsJson);
    // END SNIPPET: devGuide15

    return bulkOperationResultJson;
  }

  public PagingItemListJson<ActivityLogRecordJson> getActivityLog()
  {
    // START SNIPPET: devGuide13

    Files files = apiSession.getFilesResource();

    GetDocumentActivityLogRequestJson getDocumentActivityLogRequestJson =
        new GetDocumentActivityLogRequestJson();

    // Indicates if only the last action for a user should be retrieved
    getDocumentActivityLogRequestJson.setLastActionPerUser(false);

    // Indicates the page number to fetch of a multipage response
    getDocumentActivityLogRequestJson.setPageNumber(1);

    // The number of items to fetch per page
    getDocumentActivityLogRequestJson.setPageSize(25);

    // Call the get activity method
    PagingItemListJson<ActivityLogRecordJson> result =
        files.getActivityLogV30(getDocumentActivityLogRequestJson);
    // END SNIPPET: devGuide13

    return result;
  }
}
